
// tmp_opengl.cpp : Defines the entry point for the application.
//
// load two different textures
//
// -------------------N.B.------------
// You must tell your linker to link to the openGl libraries:
// ++++++++++++++++ 
// Visual C++
// ++++++++++++++++
// In Visual C++ go to Project, Settings, and then click on the LINK tab. 
// Under "Object/Library Modules" at the beginning of the line (before kernel32.lib) add OpenGL32.lib GLu32.lib and GLaux.lib. 
// Once you've done this click  OK. 
//
// pre-complied header for Visual C++, comment this out for other compilers
//#include "stdafx.h"
//
// ++++++++++++++++
// Dev-C++
// ++++++++++++++++
// In DevC++ select Project_Menu -> Project_options -> Parameters_Tab, type:
// -lopengl32 -lglu32 -lglaux
// in the "linker Box"
//
// Other Compilers will have diferent methods for specifing the link files



//////////////////////////////////////////////////////////////////////////////
//INCLUDES
//////////////////////////////////////////////////////////////////////////////
#define WIN32_LEAN_AND_MEAN 

#include <windows.h>   
#include <gl/gl.h> //Standard OpenGL Header
#include <gl/glu.h> //OpenGL utilities

//////////////////////////////////////////////////////////////////////////////
//DEFINES
//////////////////////////////////////////////////////////////////////////////
//name for our window class
#define WINDOWCLASS "win32OpenGL"
//title of the application
#define WINDOWTITLE "OpenGL Textures"

//////////////////////////////////////////////////////////////////////////////
//PROTOTYPES
//////////////////////////////////////////////////////////////////////////////
bool Prog_Init();//game data initalizer
void Prog_Loop();//main game loop
void Prog_Done();//game clean up

//////////////////////////////////////////////////////////////////////////////
//GLOBALS
//////////////////////////////////////////////////////////////////////////////
HINSTANCE hInstMain=NULL;//main application handle
HWND hWndMain=NULL;//handle to our main window
HDC g_hdc; 	// global handle to device context
HGLRC g_hrc; 	// global handle to rendering context

int num_textures=0;
int cur_text=0;

void SetupPixelFormat(HDC hdc){
	int nPixelFormat;

	  // set the pixel format we want
	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),	// size of structure
		1,                              // default version
		PFD_DRAW_TO_WINDOW |            // window drawing support
		PFD_SUPPORT_OPENGL |            // OpenGL support
		PFD_DOUBLEBUFFER,               // double buffering support
		PFD_TYPE_RGBA,                  // RGBA color mode
		32,                           // 32 bit color mode
		0, 0, 0, 0, 0, 0,               // ignore color bits, non-palettized mode
		0,                              // no alpha buffer
		0,                              // ignore shift bit
		0,                              // no accumulation buffer
		0, 0, 0, 0,                     // ignore accumulation bits
		16,                             // 16 bit z-buffer size
		0,                              // no stencil buffer
		0,                              // no auxiliary buffer
		PFD_MAIN_PLANE,                 // main drawing plane
		0,                              // reserved
		0, 0, 0 };                      // layer masks ignored

	// choose best matching pixelformat
	nPixelFormat=ChoosePixelFormat(hdc, &pfd);

	if (nPixelFormat == 0) {
		MessageBox(NULL, "ChoosePixelFormat() failed:  "
		   "Cannot find a suitable pixel format.", "Error", MB_OK); 
		return ;
    } 
 
	//set pixel format to device context
    if (SetPixelFormat(hdc,nPixelFormat,&pfd)== FALSE) {
		MessageBox(NULL, "SetPixelFormat() failed:  "
		   "Cannot set format specified.", "Error", MB_OK);
		return ;
    } 

	
}

void GLResize(int width,int height){

			if(height==0){height=1;} // prevent divide by zero

			glViewport(0,0,width,height);  // set new viewport size

			glMatrixMode(GL_PROJECTION); // reset projection matrix
			glLoadIdentity();

			//calculate new prespective and aspect ratio
			gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,1.0f,1000.0f);

			glMatrixMode(GL_MODELVIEW); // reset modelview matrix
			glLoadIdentity();
}


//////////////////////////////////////////////////////////////////////////////
//WINDOWPROC
//////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK TheWindowProc(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	//which message did we get?
	switch(uMsg)
	{
		case WM_CREATE: //window is being created
		{
			return 0;
		}break;

		case WM_DESTROY://the window is being destroyed
		{
			//deselect RC and delete
			wglMakeCurrent(g_hdc,NULL);
			wglDeleteContext(g_hrc);
			
			//tell the application we are quitting
			PostQuitMessage(0);
			//handled message, so return 0
			return(0);
		}break;
		
		case WM_PAINT://the window needs repainting
		{
			//a variable needed for painting information
			PAINTSTRUCT ps;
			
			//start painting
			HDC hdc=BeginPaint(hwnd,&ps);

			/////////////////////////////
			//painting code would go here
			/////////////////////////////

			//end painting
			EndPaint(hwnd,&ps);
						
			//handled message, so return 0
			return(0);
		}break;
	
		case WM_SIZE:
		{	
			GLResize(LOWORD(lParam), HIWORD(lParam));
			return(0);
		}break;
	
	}

	//pass along any other message to default message handler
	return(DefWindowProc(hwnd,uMsg,wParam,lParam));
}


//////////////////////////////////////////////////////////////////////////////
//WINMAIN
//////////////////////////////////////////////////////////////////////////////
int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nShowCmd)
{
	//assign instance to global variable
	hInstMain=hInstance;

	//create window class
	WNDCLASSEX wcx;

	//set the size of the structure
	wcx.cbSize=sizeof(WNDCLASSEX);

	//class style
	wcx.style=CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;

	//window procedure
	wcx.lpfnWndProc=TheWindowProc;

	//class extra
	wcx.cbClsExtra=0;

	//window extra
	wcx.cbWndExtra=0;

	//application handle
	wcx.hInstance=hInstMain;

	//icon
	wcx.hIcon=LoadIcon(NULL,IDI_APPLICATION);

	//cursor
	wcx.hCursor=LoadCursor(NULL,IDC_ARROW);

	//background color
	wcx.hbrBackground=NULL;

	//menu
	wcx.lpszMenuName=NULL;

	//class name
	wcx.lpszClassName=WINDOWCLASS;

	//small icon
	wcx.hIconSm=NULL;

	//register the window class, return 0 if not successful
	if(!RegisterClassEx(&wcx)) return(0);

	DWORD dwExStyle;
	DWORD dwStyle;

	int width=800,height=600;
	
	dwExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;
	dwStyle = WS_OVERLAPPEDWINDOW;
	

	// set up the window we're rendering to so that the top left corner is at (0,0)
	// and the bottom right corner is (height,width)
	RECT  windowRect;
	windowRect.left = 0;
	windowRect.right = (LONG) width;
	windowRect.top = 0;
	windowRect.bottom = (LONG) height;

	// change the size of the rect to account for borders, etc. set by the style
	AdjustWindowRectEx(&windowRect, dwStyle, FALSE, dwExStyle);

	//create main window
	hWndMain=CreateWindowEx(dwExStyle,
				WINDOWCLASS,WINDOWTITLE, 
				WS_BORDER | WS_SYSMENU | WS_VISIBLE |WS_CLIPCHILDREN |
					WS_CLIPSIBLINGS |dwStyle,
				0,0,
				windowRect.right - windowRect.left, // width
                windowRect.bottom - windowRect.top, // height
				NULL,NULL,hInstMain,NULL
				);

	//error check
	if(!hWndMain) return(0);

	g_hdc=GetDC(hWndMain); // get window DC

	SetupPixelFormat(g_hdc); //

	g_hrc=wglCreateContext(g_hdc); // create a rendering context
	wglMakeCurrent(g_hdc,g_hrc);// make it current

	ShowWindow(hWndMain,SW_SHOW);
	
	UpdateWindow(hWndMain);

	GLResize(width,height);

	//if program initialization failed, then return with 0
	if(!Prog_Init()) return(0);

	//message structure
	MSG msg;

	//message pump
	for(;;)	
	{
		//look for a message
		if(PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{
			//there is a message

			//check that we arent quitting
			if(msg.message==WM_QUIT) break;
			
			//translate message
			TranslateMessage(&msg);

			//dispatch message
			DispatchMessage(&msg);
		}

		//run main game loop
		Prog_Loop();
	}
	
	//clean up program data
	Prog_Done();

	//return the wparam from the WM_QUIT message
	return(msg.wParam);
}

void CheckGL(){
	switch(glGetError()){
		case GL_NO_ERROR:
			//MessageBox(hWndMain,"GL_NO_ERROR","Error",MB_OK);
			break;
		case GL_INVALID_ENUM:
			MessageBox(hWndMain,"GL_INVALID_ENUM","Error",MB_OK);
			break;
		case GL_INVALID_VALUE:
			MessageBox(hWndMain,"GL_INVALID_VALUE","Error",MB_OK);
			break;
		case GL_INVALID_OPERATION:
			MessageBox(hWndMain,"GL_INVALID_OPERATION","Error",MB_OK);
			break;
		case GL_STACK_OVERFLOW:
			MessageBox(hWndMain,"GL_STACK_OVERFLOW","Error",MB_OK);
			break;
		case GL_STACK_UNDERFLOW:
			MessageBox(hWndMain,"GL_STACK_UNDERFLOW","Error",MB_OK);
			break;
		case GL_OUT_OF_MEMORY:
			MessageBox(hWndMain,"GL_OUT_OF_MEMORY","Error",MB_OK);
			break;
	}
}

////// Defines
#define BITMAP_ID 0x4D42		// the universal bitmap ID
#include <stdio.h>
#include <stdlib.h>

////// Texture Information
unsigned int		texture[8];
// LoadBitmapFile
// desc: Returns a pointer to the bitmap image of the bitmap specified
//       by filename. Also returns the bitmap header information.
//		 No support for 8-bit bitmaps.
unsigned char *LoadBitmapFile(char *filename, BITMAPINFOHEADER *bitmapInfoHeader)
{
	FILE *filePtr;							// the file pointer
	BITMAPFILEHEADER	bitmapFileHeader;		// bitmap file header
	unsigned char		*bitmapImage;			// bitmap image data
	unsigned int		imageIdx = 0;		// image index counter
	unsigned char		tempRGB;				// swap variable

	// open filename in "read binary" mode
	filePtr = fopen(filename, "rb");
	if (filePtr == NULL)
		return NULL;

	// read the bitmap file header
	fread(&bitmapFileHeader, sizeof(BITMAPFILEHEADER), 1, filePtr);
	
	// verify that this is a bitmap by checking for the universal bitmap id
	if (bitmapFileHeader.bfType != BITMAP_ID)
	{
		fclose(filePtr);
		return NULL;
	}

	// read the bitmap information header
	fread(bitmapInfoHeader, sizeof(BITMAPINFOHEADER), 1, filePtr);

	// move file pointer to beginning of bitmap data
	fseek(filePtr, bitmapFileHeader.bfOffBits, SEEK_SET);

	// allocate enough memory for the bitmap image data
	bitmapImage = new unsigned char[bitmapInfoHeader->biSizeImage];

	// verify memory allocation
	if (!bitmapImage)
	{
		fclose(filePtr);
		return NULL;
	}

	// read in the bitmap image data
	int count=fread(bitmapImage, 1, bitmapInfoHeader->biSizeImage, filePtr);
	
	// make sure bitmap image data was read
	if (count == 0 || ferror(filePtr))
	{
		fclose(filePtr);	
		return NULL;
	}

	// swap the R and B values to get RGB since the bitmap color format is in BGR format 
	for (imageIdx = 0; imageIdx < bitmapInfoHeader->biSizeImage; imageIdx+=3)
	{
		tempRGB = bitmapImage[imageIdx];
		bitmapImage[imageIdx] = bitmapImage[imageIdx + 2];
		bitmapImage[imageIdx + 2] = tempRGB;
	}

	// close the file and return the bitmap image data
	fclose(filePtr);

	return bitmapImage;
}

void LoadTexture(char* str){
	
	// load our bitmap file
	BITMAPINFOHEADER	bitmapInfoHeader;	// bitmap info header
	unsigned char*		bitmapData;			// the texture data

	
	// load our bitmap file
	
	bitmapData = LoadBitmapFile(str, &bitmapInfoHeader);
	if(!bitmapData)
		MessageBox(hWndMain,"Bitmap reading error","Error",MB_OK);

	glBindTexture(GL_TEXTURE_2D, texture[num_textures++]);       // enable our texture object
	

	// build mippaps for texture 2, need these for trilinear
	gluBuild2DMipmaps( GL_TEXTURE_2D, 3, bitmapInfoHeader.biWidth, bitmapInfoHeader.biHeight,
                   GL_RGB, GL_UNSIGNED_BYTE, bitmapData );


	CheckGL(); // just check for problems//
	delete [] bitmapData;
}


//////////////////////////////////////////////////////////////////////////////
//INITIALIZATION
//////////////////////////////////////////////////////////////////////////////
int font_list_base_3d=2000; // set the start of the display lists for the 2d font
bool Prog_Init()
{
	////////////////////////////////////
	//your initialization code goes here
	////////////////////////////////////

	glClearColor(0.05f, 0.05f, 0.1f, 0.0f);		// clear to background black

	glShadeModel(GL_SMOOTH);					// use smooth shading
	glEnable(GL_DEPTH_TEST);					// hidden surface removal
	glPolygonMode(GL_FRONT,GL_FILL);

	glGenTextures(8, texture);                  // generate texture object names

	// load our bitmap file

	LoadTexture("texturehatch.bmp");
	LoadTexture("brick.bmp");
	LoadTexture("text.bmp");
	LoadTexture("bigtext.bmp");
	LoadTexture("smallbrick.bmp");
	LoadTexture("lotus05.bmp");

/*
	// load our bitmap file
	BITMAPINFOHEADER	bitmapInfoHeader;	// bitmap info header
	unsigned char*		bitmapData;			// the texture data

	
	bitmapData = LoadBitmapFile("texturehatch.bmp", &bitmapInfoHeader);
	if(!bitmapData)
		MessageBox(hWndMain,"Bitmap reading error","Error",MB_OK);


	//set trilinear sampling for texture 2
	glBindTexture(GL_TEXTURE_2D, texture[num_textures++]);       // enable our texture object
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

	// build mippaps for texture 2, need these for trilinear
	gluBuild2DMipmaps( GL_TEXTURE_2D, 3, bitmapInfoHeader.biWidth, bitmapInfoHeader.biHeight,
                   GL_RGB, GL_UNSIGNED_BYTE, bitmapData );


	CheckGL(); // just check for problems//
	delete [] bitmapData;
	
	bitmapData = LoadBitmapFile("brick.bmp", &bitmapInfoHeader);
	
	if(!bitmapData)
		MessageBox(hWndMain,"Bitmap reading error","Error",MB_OK);


	//set trilinear sampling for texture 2
	glBindTexture(GL_TEXTURE_2D, texture[num_textures++]);       // enable our texture object
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

	// build mippaps for texture 2, need these for trilinear
	gluBuild2DMipmaps( GL_TEXTURE_2D, 3, bitmapInfoHeader.biWidth, bitmapInfoHeader.biHeight,
                   GL_RGB, GL_UNSIGNED_BYTE, bitmapData );


	CheckGL(); // just check for problems//bitmapData = LoadBitmapFile("brick.bmp", &bitmapInfoHeader);
	delete [] bitmapData;
	
	bitmapData = LoadBitmapFile("text.bmp", &bitmapInfoHeader);
	if(!bitmapData)
		MessageBox(hWndMain,"Bitmap reading error","Error",MB_OK);


	//set trilinear sampling for texture 2
	glBindTexture(GL_TEXTURE_2D, texture[num_textures++]);       // enable our texture object
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

	// build mippaps for texture 2, need these for trilinear
	gluBuild2DMipmaps( GL_TEXTURE_2D, 3, bitmapInfoHeader.biWidth, bitmapInfoHeader.biHeight,
                   GL_RGB, GL_UNSIGNED_BYTE, bitmapData );


	CheckGL(); // just check for problems

	delete [] bitmapData;
	bitmapData = LoadBitmapFile("smallbrick.bmp", &bitmapInfoHeader);
	if(!bitmapData)
		MessageBox(hWndMain,"Bitmap reading error","Error",MB_OK);


	//set trilinear sampling for texture 2
	glBindTexture(GL_TEXTURE_2D, texture[num_textures++]);       // enable our texture object
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

	// build mippaps for texture 2, need these for trilinear
	gluBuild2DMipmaps( GL_TEXTURE_2D, 3, bitmapInfoHeader.biWidth, bitmapInfoHeader.biHeight,
                   GL_RGB, GL_UNSIGNED_BYTE, bitmapData );


	CheckGL(); // just check for problems
	delete [] bitmapData;

*/

	//create fonts
	HFONT font = CreateFont(40, 0, 0, 0,
					   FW_NORMAL, FALSE, FALSE, FALSE,
					   ANSI_CHARSET, 0,
						0,0,0,"Arial"); // can only use true type fonts

	SelectObject(g_hdc, font);
	wglUseFontOutlines(g_hdc,0,127,font_list_base_3d, 0.0f, 0.1f,WGL_FONT_POLYGONS,NULL); 
	


	return(true);//return success
}

//////////////////////////////////////////////////////////////////////////////
//CLEANUP
//////////////////////////////////////////////////////////////////////////////
void Prog_Done()
{
	//////////////////////////
	//clean up code goes here
	//////////////////////////

}


//////////////////////////////////////////////////////////////////////////////
//MAIN GAME LOOP


void Prog_Loop()
{
	///////////////////////////
	//main game logic goes here
	///////////////////////////
	

	static float angle =5.0f; // angle of rotation (in degrees)
	static float dist=0.0f;
	
	//check for keys
	if(GetAsyncKeyState(VK_LEFT)){
		angle+=0.5f; // increase angle of rotation
	}
	if(GetAsyncKeyState(VK_RIGHT)){
		angle-=0.5f; // increase angle of rotation
	}
	if(GetAsyncKeyState(VK_UP)){
		dist+=0.05;
	}
	if(GetAsyncKeyState(VK_DOWN)){
		dist-=0.05;
	}

	static keyup=true;

	if(GetAsyncKeyState(VK_SPACE)){
		if(keyup){ // only change if the key has been released since last change
			cur_text=(cur_text+1)%num_textures;
			keyup=false;
		}
	}
	else{
		keyup=true;
	}


	// Prepare to draw
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);//clear buffers
	glMatrixMode(GL_MODELVIEW); // reset modelview matrix
	glLoadIdentity();
	glEnable(GL_TEXTURE_2D);					// enable 2D texturing

 	// Display instructions
	glPushMatrix();
	glTranslatef(-0.8f,-1.1f,-3.0f); 		// move it back  a bit
	glScalef(0.1f,0.1f,0.1f);
	char str[]="Use up/down/left/right to move samples";
	glDisable(GL_TEXTURE_2D);
	glCallLists(strlen(str), GL_UNSIGNED_BYTE, str); // 12 characters to be displayed
	glEnable(GL_TEXTURE_2D);
	glPopMatrix();
	glPushMatrix();
	glTranslatef(-0.8f,-1.2f,-3.0f); 		// move it back  a bit
	glScalef(0.1f,0.1f,0.1f);
	char str2[]="Use 'space bar' to change textures";
	glDisable(GL_TEXTURE_2D);
	glCallLists(strlen(str2), GL_UNSIGNED_BYTE, str2); // 12 characters to be displayed
	glEnable(GL_TEXTURE_2D);
	glPopMatrix();
	

	//position camera
	glTranslatef(0.0f,0.0f,-8.0f); 		// move modeling coords  back 3units.
	glRotatef(10, 1.0f, 0.0f, 0.0f); // rotate everything around y-axis, 
	glTranslatef(0.0f,0.0f,dist); 		// move modeling coords  back 3units.

	glColor3f(1.0f,1.0f,1.0f); // set current color to white (important), try red to see what happens

	
	//========================
	//draw quad with point sampling
	//========================
	glBindTexture(GL_TEXTURE_2D, texture[cur_text]); // select texture
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	
	glPushMatrix();
	glTranslatef(-1.0f,1.0f,0.0f); 		// move it to the left
	glRotatef(-angle, 0.0f, 1.0f, 0.0f); // rotate picture around y-axis, 
	
	glPushMatrix();
	glDisable(GL_TEXTURE_2D);
	glTranslatef(-0.99f,1.1f,0.0f); 		// position the text
	glScalef(0.4f,0.4f,0.4f);				// text is to big ,so shrink it
	glCallLists(12, GL_UNSIGNED_BYTE, "Point Sampled"); // 12 characters to be displayed
	glEnable(GL_TEXTURE_2D);
	glPopMatrix();

	//draw the quad
	//give each vertex a texture coordinate
	glBegin(GL_QUADS);
		glTexCoord2i(1,1);	glVertex3i(1,1,0);
		glTexCoord2i(1,0);	glVertex3i(1,-1,0);
		glTexCoord2i(0,0);	glVertex3i(-1,-1,0);
		glTexCoord2i(0,1);	glVertex3i(-1,1,0);
	glEnd();
	
	glPopMatrix();  // undo the spinning transformations
	
	
	//========================
	//draw quad with bilinear
	//========================
	glBindTexture(GL_TEXTURE_2D, texture[cur_text]); // select texture
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	glPushMatrix();
	glTranslatef(1.0f,1.0f,0.0f); 		// move it to the right 
	glRotatef(-angle, 0.0f, 1.0f, 0.0f); // rotate picture around y-axis, 
	
	glPushMatrix();
	glTranslatef(-0.5f,1.1f,0.0f); 		// position the text
	glScalef(0.4f,0.4f,0.4f);			// text is to big ,so shrink it
	glColor3b(127,127,127);
	glDisable(GL_TEXTURE_2D);
	glCallLists(8, GL_UNSIGNED_BYTE, "Bilinear"); // 12 characters to be displayed
	glEnable(GL_TEXTURE_2D);
	glPopMatrix();

	// draw the quad
	//give each vertex a texture coordinate
	glBegin(GL_QUADS);
		glTexCoord2i(1,1);	glVertex3i(1,1,0);
		glTexCoord2i(1,0);	glVertex3i(1,-1,0);
		glTexCoord2i(0,0);	glVertex3i(-1,-1,0);
		glTexCoord2i(0,1);	glVertex3i(-1,1,0);
	glEnd();
	
	glPopMatrix();  // undo the spinning transformations

	//========================
	//draw quad with bilinear mipmap
	//========================
	glBindTexture(GL_TEXTURE_2D, texture[cur_text]); // select texture
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);

	glPushMatrix();
	
	glTranslatef(-1.02f,-1.02f,0.0f); 		// move it to the right
	glRotatef(-angle, 0.0f, 1.0f, 0.0f); // rotate picture around y-axis, 
	
	//draw the quad
	//give each vertex a texture coordinate
	glListBase(font_list_base_3d); 
	glPushMatrix();
	glTranslatef(-1.3f,-1.3f,0.0f); 		// position the text
	glScalef(0.4f,0.4f,0.4f);			// text is to big ,so shrink it
	glDisable(GL_TEXTURE_2D);
	glCallLists(15, GL_UNSIGNED_BYTE, "Bilinear Mipmap"); // 12 characters to be displayed
	glEnable(GL_TEXTURE_2D);
	glPopMatrix();
	glBegin(GL_QUADS);
		glTexCoord2i(1,1);	glVertex3i(1,1,0);
		glTexCoord2i(1,0);	glVertex3i(1,-1,0);
		glTexCoord2i(0,0);	glVertex3i(-1,-1,0);
		glTexCoord2i(0,1);	glVertex3i(-1,1,0);
	glEnd();
	
	
	glPopMatrix();  // undo the spinning transformations
	
	//========================
	//draw quad with trilinear mipmap
	//========================
	glBindTexture(GL_TEXTURE_2D, texture[cur_text]); // select texture
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

	glPushMatrix();
	
	glTranslatef(1.02f,-1.02f,0.0f); 		// move it to the right
	glRotatef(-angle, 0.0f, 1.0f, 0.0f); // rotate picture around y-axis, 
	
	//draw the quad
	//give each vertex a texture coordinate
	glListBase(font_list_base_3d); 
	glPushMatrix();
	glTranslatef(-0.3f,-1.3f,0.0f); 		// position the text
	glScalef(0.4f,0.4f,0.4f);			// text is to big ,so shrink it
	glDisable(GL_TEXTURE_2D);
	glCallLists(9, GL_UNSIGNED_BYTE, "Trilinear"); // 12 characters to be displayed
	glEnable(GL_TEXTURE_2D);
	glPopMatrix();
	glBegin(GL_QUADS);
		glTexCoord2i(1,1);	glVertex3i(1,1,0);
		glTexCoord2i(1,0);	glVertex3i(1,-1,0);
		glTexCoord2i(0,0);	glVertex3i(-1,-1,0);
		glTexCoord2i(0,1);	glVertex3i(-1,1,0);
	glEnd();
	
	
	glPopMatrix();  // undo the spinning transformations


	SwapBuffers(g_hdc); // bring back buffer foreground
}

